package org.warp.jcwdb;

import java.util.ArrayList;

import it.unimi.dsi.fastutil.longs.LongArrayList;

public class DBLightListParser<T> extends DBTypeParserImpl<LightList<T>> {
	private final JCWDatabase db;

	public DBLightListParser(JCWDatabase db) {
		this.db = db;
	}
	
	public DBReader<LightList<T>> getReader() {
		return (i, size) -> {
			LongArrayList internalList = new LongArrayList();
			long max = size / Long.BYTES;
			for (int item = 0; item < max; item++){
				long itm = i.readLong();
				internalList.add(itm);
			}
			return new LightList<T>(db, internalList);
		};
	}

	public DBDataOutput<LightList<T>> getWriter(final LightList<T> value) {
		final int elementsCount = value.size();
		return DBDataOutput.create((o) -> {
			LongArrayList list = value.internalList;
			for (int i = 0; i < elementsCount; i++) {
				o.writeLong(list.getLong(i));
			}
		}, DBStandardTypes.LIGHT_LIST, elementsCount * Long.BYTES, calculateHash(value));
	}

	@Override
	public long calculateHash(LightList<T> value) {
		return value.internalList.hashCode();
	}
}
